// Copyright (c) 2010 The Chromium OS Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef _CRASH_REPORTER_KERNEL_COLLECTOR_H_
#define _CRASH_REPORTER_KERNEL_COLLECTOR_H_

#include <pcrecpp.h>

#include <string>

#include "base/file_path.h"
#include "crash-reporter/crash_collector.h"
#include "gtest/gtest_prod.h"  // for FRIEND_TEST

class FilePath;

// Kernel crash collector.
class KernelCollector : public CrashCollector {
 public:
  // Enumeration to specify architecture type.
  enum ArchKind {
    archUnknown,
    archArm,
    archX86,

    archCount  // Number of architectures.
  };

  KernelCollector();

  virtual ~KernelCollector();

  void OverridePreservedDumpPath(const FilePath &file_path);

  // Enable collection.
  bool Enable();

  // Returns true if the kernel collection currently enabled.
  bool IsEnabled() {
    return is_enabled_;
  }

  // Collect any preserved kernel crash dump. Returns true if there was
  // a dump (even if there were problems storing the dump), false otherwise.
  bool Collect();

  // Compute a stack signature string from a kernel dump.
  bool ComputeKernelStackSignature(const std::string &kernel_dump,
                                   std::string *kernel_signature,
                                   bool print_diagnostics);

  // Set the architecture of the crash dumps we are looking at.
  void SetArch(enum ArchKind arch);
  enum ArchKind GetArch() { return arch_; }

  // Set the parameters for reading the crash dump.
  void SetParameters(size_t record_size, size_t mem_start, size_t mem_size);

 private:
  friend class KernelCollectorTest;
  FRIEND_TEST(KernelCollectorTest, LoadPreservedDump);
  FRIEND_TEST(KernelCollectorTest, StripSensitiveDataBasic);
  FRIEND_TEST(KernelCollectorTest, StripSensitiveDataBulk);
  FRIEND_TEST(KernelCollectorTest, StripSensitiveDataSample);
  FRIEND_TEST(KernelCollectorTest, CollectOK);

  bool LoadPreservedDump(std::string *contents);
  void StripSensitiveData(std::string *kernel_dump);

  virtual bool LoadParameters();
  bool HasMoreRecords();

  // Read a record to string, modified from file_utils since that didn't
  // provide a way to restrict the read length.
  // Return value indicates (only) error state:
  //  * false when we get an error (can't read from dump location).
  //  * true if no error occured.
  // Not finding a valid record is not an error state and is signaled by the
  // record_found output parameter.
  bool ReadRecordToString(std::string *contents,
                          unsigned int current_record,
                          bool *record_found);

  void ProcessStackTrace(pcrecpp::StringPiece kernel_dump,
                         bool print_diagnostics,
                         unsigned *hash,
                         float *last_stack_timestamp);
  bool FindCrashingFunction(pcrecpp::StringPiece kernel_dump,
                            bool print_diagnostics,
                            float stack_trace_timestamp,
                            std::string *crashing_function);
  bool FindPanicMessage(pcrecpp::StringPiece kernel_dump,
                        bool print_diagnostics,
                        std::string *panic_message);

  // Returns the architecture kind for which we are built - enum ArchKind.
  enum ArchKind GetCompilerArch(void);

  bool is_enabled_;
  size_t mem_start_;
  size_t mem_size_;
  FilePath ramoops_dump_path_;
  FilePath ramoops_record_size_path_;
  FilePath ramoops_dump_start_path_;
  FilePath ramoops_dump_size_path_;
  size_t record_size_;

  // The architecture of kernel dump strings we are working with.
  enum ArchKind arch_;
};

#endif  // _CRASH_REPORTER_KERNEL_COLLECTOR_H_
